%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% update_g_invwishart_row:
%
% Samples whole "y" from a G-Inverse Wishart distribution encoded in the
% Bartlett decomposition.
%
% Initial version: Ricardo Silva, UCL - London, April 2011
% Current version: 10/10/2011

function [nrow B g] = update_g_invwishart_row(y, U, delta, V, sp, nsp, not_y)

  % Initialization
  num_y = size(U, 1);
  B     = zeros(1, num_y);

  % Auxiliary information
  U_ss = U(sp, sp);   
  U_ns = U(nsp, sp); 
  U_nn = U(nsp, nsp); 

  M = zeros(num_y, 1);
  M(not_y) = U(not_y, not_y) \ U(not_y, y); 
  u = U(y, y) - U(y, not_y) * M(not_y);

  A  = solve_chol(chol(V(nsp, nsp)), V(nsp, sp))';   
  sub1 = U_ss - A * U_ns; 
  sub2 = U_ns' - A * U_nn;
  m  = [sub1 sub2] * [M(sp) ; M(nsp)]; 
  K_chol = chol(sub1 - sub2 * A'); % invK = U_ss - A * U_ns - U_sn * A'  + A * U_nn * A';
  v = K_chol' \ m;
  Ud = -u + U(y, y) - v' * v; % = M' * U(not_y, not_y) * M - m' * inv(invK) * m;
    
  % Sampling variance term
  a = 0.5 * (delta + (num_y - 1) + length(nsp));        
  b = 0.5 * (u + Ud);
  g = b / gamrnd(a, 1);

  % Sampling spouse/non-spouse coefficients    
  B(sp)  = K_chol \ v + sqrt(g) * K_chol \ randn(sum(sp), 1); % = randnorm(1, m * inv(invK) , [], g * inv(invK))';
  B(nsp) = -B(sp) * A;

  % Reconstructing parameters
  nrow        = zeros(1, num_y);
  nrow(not_y) = B(not_y) * V(not_y, not_y);
  nrow(y)     = g + B(not_y) * nrow(not_y)';

end
